﻿using Microsoft.Extensions.Logging;
using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.BusinessLogicsContracts;
using PlumbingRepairContracts.SearchModels;
using PlumbingRepairContracts.ViewModels;
using PlumbingRepairDataModels.Models;
using System.Reflection;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PlumbingRepairView
{
    public partial class FormStore : Form
    {
        private readonly IStoreLogic _logic;
        private readonly ILogger _logger;

        private int? _id;
        private Dictionary<int, (IWorkModel, int)> _listStores;
        public int Id { set { _id = value; } }

        public FormStore(ILogger<FormStore> logger, IStoreLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _listStores = new();
            _logic = logic;
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(NameTextBox.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (string.IsNullOrEmpty(AdressTextBox.Text))
            {
                MessageBox.Show("Заполните адрес", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Сохранение магазина");

            try
            {
                var model = new StoreBindingModel
                {
                    Id = _id ?? 0,
                    StoreName = NameTextBox.Text,
                    StoreAdress = AdressTextBox.Text,
                    OpeningDate = DateTime.SpecifyKind(OpeningDatePicker.Value.Date, DateTimeKind.Utc),
                    WorkMaxCount = (int)VolumeNumericUpDown.Value,
                    StoreWorks = _listStores
                };
                var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);

                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения магазина");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void FormStore_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                _logger.LogInformation("Загрузка магазина");
                try
                {
                    var view = _logic.ReadElement(new StoreSearchModel
                    {
                        Id = _id.Value
                    });
                    if (view != null)
                    {
                        NameTextBox.Text = view.StoreName;
                        AdressTextBox.Text = view.StoreAdress;
                        OpeningDatePicker.Text = view.OpeningDate.ToString();
                        VolumeNumericUpDown.Value = view.WorkMaxCount;
                        _listStores = view.StoreWorks ?? new Dictionary<int, (IWorkModel, int)>();
                        LoadData();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка загрузки магазина");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }
        }
        private void LoadData(bool extendDate = true)
        {
            _logger.LogInformation("Загрузка изделий магазина");
            try
            {
                if (_listStores != null)
                {
                    DataGridView.Rows.Clear();
                    foreach (var elem in _listStores)
                    {
                        DataGridView.Rows.Add(new object[] { elem.Value.Item1.WorkName, elem.Value.Item1.Price, elem.Value.Item2 });
                    }
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки изделий магазина");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
